@@ -11,7 +11,7 @@
 /*                                                                     */
 /***********************************************************************/
 
-/* $Id: stack.h,v 1.29.2.1 2006/03/01 13:46:56 xleroy Exp $ */
+/* $Id: stack.h 10315 2010-04-27 07:55:08Z xleroy $ */
 
 /* Machine-dependent interface with the asm code */
 
@@ -53,9 +53,9 @@
 #endif
 
 #ifdef TARGET_power
-#define Saved_return_address(sp) *((intnat *)((sp) - 4))
+#define Saved_return_address(sp) *((intnat *)((sp) - SIZEOF_PTR))
 #define Already_scanned(sp, retaddr) ((retaddr) & 1)
-#define Mark_scanned(sp, retaddr) (*((intnat *)((sp) - 4)) = (retaddr) | 1)
+#define Mark_scanned(sp, retaddr) (*((intnat *)((sp) - SIZEOF_PTR)) = (retaddr) | 1)
 #define Mask_already_scanned(retaddr) ((retaddr) & ~1)
 #ifdef SYS_aix
 #define Trap_frame_size 32
@@ -96,7 +96,32 @@ struct caml_context {
   value * gc_regs;              /* pointer to register block */
 };
 
+/* Structure of frame descriptors */
+
+typedef struct {
+  uintnat retaddr;
+  unsigned short frame_size;
+  unsigned short num_live;
+  unsigned short live_ofs[1];
+} frame_descr;
+
+/* Hash table of frame descriptors */
+
+extern frame_descr ** caml_frame_descriptors;
+extern int caml_frame_descriptors_mask;
+
+#define Hash_retaddr(addr) \
+  (((uintnat)(addr) >> 3) & caml_frame_descriptors_mask)
+
+extern void caml_init_frame_descriptors(void);
+extern void caml_register_frametable(intnat *);
+extern void caml_register_dyn_global(void *);
+
+extern uintnat caml_stack_usage (void);
+extern uintnat (*caml_stack_usage_hook)(void);
+
 /* Declaration of variables used in the asm code */
+extern char * caml_top_of_stack;
 extern char * caml_bottom_of_stack;
 extern uintnat caml_last_return_address;
 extern value * caml_gc_regs;
@@ -105,5 +130,4 @@ extern value caml_globals[];
 extern intnat caml_globals_inited;
 extern intnat * caml_frametable[];
 
-
 #endif /* CAML_STACK_H */
