@@ -11,7 +11,7 @@
 /*                                                                     */
 /***********************************************************************/
 
-/* $Id: backtrace.h,v 1.6 2003/12/31 14:20:35 doligez Exp $ */
+/* $Id: backtrace.h 9540 2010-01-20 16:26:46Z doligez $ */
 
 #ifndef CAML_BACKTRACE_H
 #define CAML_BACKTRACE_H
@@ -22,9 +22,12 @@ CAMLextern int caml_backtrace_active;
 CAMLextern int caml_backtrace_pos;
 CAMLextern code_t * caml_backtrace_buffer;
 CAMLextern value caml_backtrace_last_exn;
+CAMLextern char * caml_cds_file;
 
-extern void caml_init_backtrace(void);
+CAMLprim value caml_record_backtrace(value vflag);
+#ifndef NATIVE_CODE
 extern void caml_stash_backtrace(value exn, code_t pc, value * sp);
+#endif
 CAMLextern void caml_print_exception_backtrace(void);
 
 #endif /* CAML_BACKTRACE_H */
