@@ -11,7 +11,7 @@
 /*                                                                     */
 /***********************************************************************/
 
-/* $Id: io.c,v 1.72.2.2 2006/06/08 14:57:59 xleroy Exp $ */
+/* $Id: io.c 10300 2010-04-23 07:58:59Z shinwell $ */
 
 /* Buffered input/output. */
 
@@ -62,13 +62,16 @@ CAMLexport struct channel * caml_open_descriptor_in(int fd)
 
   channel = (struct channel *) caml_stat_alloc(sizeof(struct channel));
   channel->fd = fd;
-  channel->offset = lseek (fd, 0, SEEK_CUR);
+  caml_enter_blocking_section();
+  channel->offset = lseek(fd, 0, SEEK_CUR);
+  caml_leave_blocking_section();
   channel->curr = channel->max = channel->buff;
   channel->end = channel->buff + IO_BUFFER_SIZE;
   channel->mutex = NULL;
   channel->revealed = 0;
   channel->old_revealed = 0;
   channel->refcount = 0;
+  channel->flags = 0;
   channel->next = caml_all_opened_channels;
   channel->prev = NULL;
   if (caml_all_opened_channels != NULL)
@@ -110,13 +113,21 @@ CAMLexport void caml_close_channel(struct channel *channel)
 
 CAMLexport file_offset caml_channel_size(struct channel *channel)
 {
+  file_offset offset;
   file_offset end;
+  int fd;
 
-  end = lseek(channel->fd, 0, SEEK_END);
-  if (end == -1 ||
-      lseek(channel->fd, channel->offset, SEEK_SET) != channel->offset) {
+  /* We extract data from [channel] before dropping the Caml lock, in case
+     someone else touches the block. */
+  fd = channel->fd;
+  offset = channel->offset;
+  caml_enter_blocking_section();
+  end = lseek(fd, 0, SEEK_END);
+  if (end == -1 || lseek(fd, offset, SEEK_SET) != offset) {
+    caml_leave_blocking_section();
     caml_sys_error(NO_ARG);
   }
+  caml_leave_blocking_section();
   return end;
 }
 
@@ -162,7 +173,7 @@ again:
       n = 1; goto again;
     }
   }
-  if (retcode == -1) caml_sys_error(NO_ARG);
+  if (retcode == -1) caml_sys_io_error(NO_ARG);
   return retcode;
 }
 
@@ -244,7 +255,12 @@ CAMLexport void caml_really_putblock(struct channel *channel,
 CAMLexport void caml_seek_out(struct channel *channel, file_offset dest)
 {
   caml_flush(channel);
-  if (lseek(channel->fd, dest, SEEK_SET) != dest) caml_sys_error(NO_ARG);
+  caml_enter_blocking_section();
+  if (lseek(channel->fd, dest, SEEK_SET) != dest) {
+    caml_leave_blocking_section();
+    caml_sys_error(NO_ARG);
+  }
+  caml_leave_blocking_section();
   channel->offset = dest;
 }
 
@@ -265,7 +281,7 @@ CAMLexport int caml_do_read(int fd, char *p, unsigned int n)
     retcode = read(fd, p, n);
     caml_leave_blocking_section();
   } while (retcode == -1 && errno == EINTR);
-  if (retcode == -1) caml_sys_error(NO_ARG);
+  if (retcode == -1) caml_sys_io_error(NO_ARG);
   return retcode;
 }
 
@@ -339,7 +355,12 @@ CAMLexport void caml_seek_in(struct channel *channel, file_offset dest)
       dest <= channel->offset) {
     channel->curr = channel->max - (channel->offset - dest);
   } else {
-    if (lseek(channel->fd, dest, SEEK_SET) != dest) caml_sys_error(NO_ARG);
+    caml_enter_blocking_section();
+    if (lseek(channel->fd, dest, SEEK_SET) != dest) {
+      caml_leave_blocking_section();
+      caml_sys_error(NO_ARG);
+    }
+    caml_leave_blocking_section();
     channel->offset = dest;
     channel->curr = channel->max = channel->buff;
   }
@@ -473,19 +494,30 @@ CAMLprim value caml_channel_descriptor(value vchannel)
 CAMLprim value caml_ml_close_channel(value vchannel)
 {
   int result;
+  int do_syscall;
+  int fd;
 
   /* For output channels, must have flushed before */
   struct channel * channel = Channel(vchannel);
   if (channel->fd != -1){
-    result = close(channel->fd);
+    fd = channel->fd;
     channel->fd = -1;
+    do_syscall = 1;
   }else{
+    do_syscall = 0;
     result = 0;
   }
   /* Ensure that every read or write on the channel will cause an
      immediate caml_flush_partial or caml_refill, thus raising a Sys_error
      exception */
   channel->curr = channel->max = channel->end;
+
+  if (do_syscall) {
+    caml_enter_blocking_section();
+    result = close(fd);
+    caml_leave_blocking_section();
+  }
+
   if (result == -1) caml_sys_error (NO_ARG);
   return Val_unit;
 }
@@ -534,7 +566,7 @@ CAMLprim value caml_ml_flush_partial(value vchannel)
   struct channel * channel = Channel(vchannel);
   int res;
 
-  if (channel->fd == -1) CAMLreturn (Val_true);
+  if (channel->fd == -1) CAMLreturn(Val_true);
   Lock(channel);
   res = caml_flush_partial(channel);
   Unlock(channel);
@@ -546,7 +578,7 @@ CAMLprim value caml_ml_flush(value vchannel)
   CAMLparam1 (vchannel);
   struct channel * channel = Channel(vchannel);
 
-  if (channel->fd == -1) CAMLreturn (Val_unit);
+  if (channel->fd == -1) CAMLreturn(Val_unit);
   Lock(channel);
   caml_flush(channel);
   Unlock(channel);
