@@ -10,7 +10,7 @@
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: asttypes.mli,v 1.9 2003/04/25 12:27:30 xleroy Exp $ *)
+(* $Id: asttypes.mli 10250 2010-04-08 03:58:41Z garrigue $ *)
 
 (* Auxiliary a.s.t. types used by parsetree and typedtree. *)
 
@@ -33,4 +33,8 @@ type mutable_flag = Immutable | Mutable
 
 type virtual_flag = Virtual | Concrete
 
+type override_flag = Override | Fresh
+
+type closed_flag = Closed | Open
+
 type label = string
